import abc
import dataclasses
from datetime import datetime
from enum import Enum

from spadesdk.user import User


@dataclasses.dataclass
class Process:
    """
    Dataclass representing a Spade process.
    """

    code: str
    system_params: dict | None = None


@dataclasses.dataclass
class RunResult:
    """
    Base class for the result of a process run.
    """

    class Status(Enum):
        NEW = "new"
        RUNNING = "running"
        FINISHED = "finished"
        FAILED = "failed"

    class Result(Enum):
        SUCCESS = "success"
        WARNING = "warning"
        FAILED = "failed"

    process: Process
    status: Status
    result: Result | None = None
    error_message: str | None = None
    output: dict | None = None
    created_at: datetime | None = None
    user_id: int | None = None


class Executor:
    """
    Executor executes a Spade process using the run method.
    It can either directly run some code or call an external service,
    form example trigger an Airflow DAG.
    """

    @classmethod
    @abc.abstractmethod
    def run(cls, process: Process, user_params: dict, user: User, *args, **kwargs) -> RunResult:
        """
        Execute a process using the executor.

        :param process: Process to run and its system parameters
        :param user_params: User parameters - provided by the user when running the process
        :param user_id: The id of the user running the process

        :return: RunResult
        """



import abc
import typing

from spadesdk.executor import Process, RunResult


class HistoryProvider:
    """
    HistoryProvider can provide a history of process runs from an external service.
    """

    @classmethod
    @abc.abstractmethod
    def get_runs(cls, process: Process, request, *args, **kwargs) -> typing.Iterable["RunResult"]:
        """
        Get the history of a process.

        :param process: Process to get the history of
        :param request: Django request
        :param args: Additional arguments
        :param kwargs: Additional keyword arguments

        :return: Iterable of RunResult
        """



import abc
import dataclasses
from enum import Enum

from spadesdk.user import User


@dataclasses.dataclass
class File:
    """
    Dataclass representing a Spade file.
    """

    name: str
    format: str
    system_params: dict | None = None


@dataclasses.dataclass
class FileUpload:
    """
    Dataclass representing the result of a file upload.
    """

    class Result(Enum):
        SUCCESS = "success"
        WARNING = "warning"
        FAILED = "failed"

    file: File
    result: Result
    rows: int | None = None
    error_message: str | None = None
    output: dict | None = None


class FileProcessor:
    """
    FileProcessor processes a Spade file using the process method.
    """

    @classmethod
    @abc.abstractmethod
    def process(cls, file: File, filename: str, data, user_params: dict | None, user: User, *args, **kwargs) -> FileUpload:
        """
        Process a file using the file processor.

        :param file: File to process and its system parameters
        :param filename: Name of the file uploaded
        :param data: Data of the file uploaded
        :param user_params: User parameters - provided by the user when uploading the file
        """
